
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AI
CPP
Camel
Classes
Docs
Inception
InceptionClient
Overview
Predict
PredictRole
PredictionClient
PredictionGrpcCpp
PredictionLoopback
PredictionRole
Roles
SimpleTensor
Stall
TensorFlow
Testing
Tom
bin
gRPC
lib
loopback
proto
protofile
protofiles
stall
